use tokio::io::{AsyncWrite, AsyncWriteExt};
use tracing::instrument::WithSubscriber;

use lavina_core::prelude::Str;
use lavina_core::repo::Storage;
use proto_irc::client::command_args::Whois;
use proto_irc::response::{IrcResponseMessage, SendResponseBody, SendResponseMessage};

use crate::commands::whois::error::{ERR_NONICKNAMEGIVEN_431, ERR_NOSUCHNICK_401};
use crate::commands::whois::response::{RplWhoIsUser311, RPL_ENDOFWHOIS_318};
use crate::commands::Handler;

pub mod error;
pub mod response;

impl Handler<Whois> for Whois {
    async fn handle(
        &self,
        server_name: &Str,
        client: &Str,
        writer: &mut (impl AsyncWrite + Unpin),
        storage: &mut Storage,
    ) -> anyhow::Result<()> {
        match self {
            Whois::Nick(nick) => handle_nick_target(nick, None, server_name, client, writer, storage).await?,
            Whois::TargetNick(target, nick) => {
                handle_nick_target(nick, Some(target), server_name, client, writer, storage).await?
            }
            Whois::EmptyArgs => {
                IrcResponseMessage::empty_tags(
                    Some(server_name.clone()),
                    ERR_NONICKNAMEGIVEN_431::new(server_name.clone()),
                )
                .write_response(writer)
                .await?
            }
        }
        Ok(())
    }
}

async fn handle_nick_target(
    nick: &Str,
    // todo: implement logic with target
    _target: Option<&Str>,
    server_name: &Str,
    client: &Str,
    writer: &mut (impl AsyncWrite + Unpin),
    storage: &mut Storage,
) -> anyhow::Result<()> {
    if let Some(user) = storage.retrieve_user_by_name(nick).await? {
        IrcResponseMessage::empty_tags(
            Some(server_name.clone()),
            RplWhoIsUser311::new(
                client.clone(),
                nick.clone(),
                Some(Str::from(user.name.clone())),
                server_name.clone(),
            ),
        )
        .write_response(writer)
        .await?;

        IrcResponseMessage::empty_tags(
            Some(server_name.clone()),
            RPL_ENDOFWHOIS_318::new(client.clone(), user.name.clone().into()),
        )
        .write_response(writer)
        .await?
    } else {
        ERR_NOSUCHNICK_401::new(client.clone(), nick.clone()).write_response(writer).await?
    }
    Ok(())
}
